#!/usr/bin/python
# compose.py - A python class to represent a compose, and read/write XML for it.
# Author: Will Woods <wwoods@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from snake.tree import Debug, Source, Tree, IsoSet
from snake.xmlhelper import indent, ET
import os.path

class Compose(object):
    def __init__(self, **kwargs):
        self.name="" # human readable string
        self.time=0.0
        # these should be lists of the corresponding objects
        self.trees=[]
        self.isos=[]
        self.debug=[]
        self.source=[]
        if 'xml' in kwargs:
            self.parsexml(kwargs['xml'])
        elif 'dir' in kwargs:
            self.loaddir(kwargs['dir'])

    def loaddir(self,dir):
        xmlfile=os.path.join(dir,'.compose.xml')
        composeinfofile=os.path.join(dir,'.composeinfo')
        if os.path.exists(xmlfile):
            return self.parsexml(xmlfile)
        elif os.path.exists(composeinfofile):
            return self.parsecomposeinfo(composeinfofile)

    def parsecomposeinfo(self,composeinfo):
        f=open(composeinfo)
        self.name=os.path.basename(os.path.dirname(composeinfo))
        for line in f.readlines():
            (type,info) = line.split(":")
            if type == 'tree':
                dir = os.path.join(os.path.dirname(composeinfo),info.strip())
                t=Tree()
                t.reldir = info.strip()
                t.compose=self.name
                t.loaddir(dir)
                self.trees.append(t)
            # FIXME: add other types

    def parsexml(self,xml):
        xmldir=os.path.dirname(xml)
        elemtree = ET.parse(xml)
        r=elemtree.getroot()
        self.name=r.get('name')
        self.time=r.get('time')
        typemap={'debug' :(Debug,self.debug),
                 'source':(Source,self.source),
                 'tree'  :(Tree,self.trees),
                 'cdset' :(IsoSet,self.isos),
                 'dvdset':(IsoSet,self.isos)}
        for (type, (obj, set)) in typemap.items():
            for e in r.findall(type):
                c=obj()
                c.arch=e.get('arch')
                c.variant=e.get('variant') or ""
                c.reldir=e.text
                try:
                    c.parsexml(os.path.join(xmldir,c.reldir,'.'+type+'.xml'))
                    c.compose=self.name
                except:
                    pass
                set.append(c)
        return elemtree

    def element(self):
        compose = ET.Element('compose',name=self.name,time=str(self.time))
        for type, set in {'debug':self.debug,'source':self.source,
                          'tree':self.trees,'isos':self.isos}.items():
            for i in set:
                elem = ET.SubElement(compose,type,arch=i.arch)
                if type in ('tree','isos') and i.variant:
                    elem.attrib['variant'] = i.variant
                elem.text=i.reldir
        return compose

    def xml(self):
        return ET.tostring(indent(self.element()))

    def __repr__(self):
        return "Compose(%s)" % self.name

    def __str__(self):
        return "%s%s" % (self.name, str(self.trees))
