#!/usr/bin/python
# config.py - configuration for snake stuff
#
# Copyright 2007 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from ConfigParser import ConfigParser
from socket import gethostname,gethostbyaddr,gaierror

def getipaddr():
    try:
        return gethostbyaddr(gethostname())[-1][0]
    except gaierror:
        '''on some systems the hostname may not be a reverse mappable'''
        return "0.0.0.0"

DEFAULT_CONFIGFILE = "/etc/snake.conf"
DEFAULT_HOSTNAME = gethostname()
DEFAULT_IPADDR = getipaddr()
DEFAULT_PORT = 2903 # used by "suitcase".. but do we care? 

# Default configs
defaults={
    "server": {
        "hostname":             DEFAULT_HOSTNAME,
        "listen":               DEFAULT_IPADDR,
        "port":                 DEFAULT_PORT,
        "zeroconf":             "yes", 
        "http_dir":             "/var/www/html/snake", 
        "http_root":            "",
        "xmlcachedir":          "/var/cache/snake",
        "plugindir":            "/var/lib/snake/plugins",
        "templatedir":          "/var/lib/snake/templates",
        "machinedbdir":         "/var/lib/snake/machinedb",
        "logfile":              "/var/log/snake",
        "xmlcache_refresh":     "10",
        "puthandler_enabled":   "no"
    }
}

def getconfig(section="server", filename=DEFAULT_CONFIGFILE):
    parser = ConfigParser(defaults[section])
    parser.read(filename)
    configlist = parser.items(section,True)
    return dict(configlist)

if __name__ == '__main__':
    import sys
    c=getconfig(sys.argv[1])
    print c[sys.argv[2]]
