#!/usr/bin/python
#
# Copyright 2007 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from lxml import etree

class LabQuery(object):
    def __init__(self, index):
        self.et = etree.parse(index)
        self.et.xinclude()
        self.root = self.et.getroot()

    def query(self, expression):
        return self.root.xpath(expression)

    def nick_to_id(self, nickname):
        return self.root.xpath('/lab/machine[nickname = "%s"]/@id' % nickname)

    def fqdn_to_id(self, fqdn):
        return self.root.xpath('/lab/machine[fqdn = "%s"]/@id' % fqdn)

    def mac_to_id(self, mac):
        return self.root.xpath('/lab/machine[fingerprint = "%s"]/@id' % mac)

    def id_to_info(self, id):
        info = {}
        for node in self.query('/lab/machine[@id = "%s"]' % id):
            info['nickname'] = node.xpath('nickname')[0].text
            info['fqdn'] = node.xpath('fqdn')[0].text
            info['bootargs'] = node.xpath('bootargs')[0].text
            info['fingerprints'] = [i.text for i in node.xpath('fingerprint')]
            info['ksdevice'] = node.xpath('ksdevice')[0].text
        return info
