#!/usr/bin/python
#
# Copyright 2007 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import sys
import os
import logging
import logging.handlers

# yanked almost directly from yum/logginglevels.py
QUIET = -1
INFO = 0
INFO_1 = 1
INFO_2 = 2
DEBUG = 3

logging.addLevelName(INFO_1, "INFO_1")
logging.addLevelName(INFO_2, "INFO_2")

def setup_logger(logLevel=None, logfile=None):
    '''setup logging'''

    # create logger
    logging.basicConfig()
    logger = logging.getLogger("snake")
    logger.propagate = False

    # log output to stdout by default
    plainformatter = logging.Formatter("%(message)s")
    console_stdout = logging.StreamHandler(sys.stdout)
    console_stdout.setFormatter(plainformatter)
    logger.addHandler(console_stdout)

    # define a logging instance for the /var/log/snake.log logfile
    if logfile:
        add_file_log(logfile)

    # setup default logLevel
    if logLevel is not None:
        setLogLevel(logLevel)

    return logger

def logLevelFromDebugLevel(debug_level):
    """ Convert an local log levels to something the logging module understands """
    debug_table = { QUIET  : 100,
                    INFO   : logging.INFO, 
                    INFO_1 : logging.INFO - 1, 
                    INFO_2 : logging.INFO - 2, 
                    DEBUG  : logging.DEBUG, 
                  }

    return __convertLevel(debug_level, debug_table)

def __convertLevel(level, table):
    """ Convert snake logging levels using a lookup table. """
    # Look up level in the table.
    try:
        new_level = table[level]
    except KeyError:
        print "level not found"
        keys = table.keys()
        # We didn't find the level in the table, check if it's smaller
        # than the smallest level
        if level < keys[0]:
            new_level = table[keys[0]]
        # Nope. So it must be larger.
        else:
            new_level =  table[keys[-2]]

    return new_level

def add_file_log(logfile):
    '''attach a file logger to the snake log handler'''
    # TODO: When python's logging config parser doesn't blow up
    # when the user is non-root, put this in the config file.
    # syslog-style log
    try:
        filelogger = logging.getLogger("snake.filelogging")
        filelogger.propagate = False
        filehandler = logging.FileHandler(logfile, 'a')
        formatter = logging.Formatter("%(asctime)s %(name)s: %(levelname)s: %(message)s", "%b %d %H:%M:%S")
        filehandler.setFormatter(formatter)
        filelogger.addHandler(filehandler)

        # add file logger to the snake logger
        logger = logging.getLogger("snake")
        logger.addHandler(filelogger)
    # FIXME: XXX: A global exception catch? That seems unlikely.
    except Exception:
        logging.getLogger("snake").critical('Cannot open logfile %s', logfile)

def detach_console(logger=None):
    '''detach the console logger from the root console'''
    if not logger:
        logger = logging.getLogger("snake")

    for hdlr in logger.handlers:
        if isinstance(hdlr, logging.StreamHandler):
            logger.removeHandler(hdlr)

    # route stdout/stderr to /dev/null
    sys.stdin = open('/dev/null', 'r')
    sys.stdout = open('/dev/null', 'w')
    sys.stderr = open('/dev/null', 'w')

def setLogLevel(level):
    converted_level = logLevelFromDebugLevel(level)
    logging.getLogger("snake").setLevel(converted_level)
