#!/usr/bin/python
# machine.py - a Machine object for SNAKE
# Will Woods <wwoods@redhat.com> Nov. 10, 2006
#
# Copyright 2007 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from snake.xmlhelper import indent, ET

class Machine(object):
    def __init__(self,**kwargs):
        self.id=""
        self.nickname=""
        self.fqdn=""
        self.archlist=[]
        self.bootargs=""
        self.fingerprints=[]
        self.ksdevice=""
        self.hwinfo=[]
        if 'id' in kwargs:
            self.id = kwargs['id']
        if 'nickname' in kwargs:
            self.nickname = kwargs['nickname']
        if 'xml' in kwargs:
            self.parse(kwargs['xml'])

    def parse(self,xml):
        elemtree = ET.parse(xml)
        r=elemtree.getroot()
        self.id = r.get('id')                      # machine id
        self.nickname = r.find('nickname').text    # nickname
        self.fqdn = r.find('fqdn').text            # fqdn
        self.bootargs=r.find('bootargs').text      # bootargs
        for a in r.findall('arch'):                # arch(es)
            self.archlist.append(a.text)
        for a in r.findall('fingerprint'):         # fingerprints
            self.fingerprints.append(a.text)
        self.ksdevice = r.find('ksdevice').text    # ksdevice
        return elemtree

    def element(self,klist=()):
        def _traverse_element(self,child,elem):
            subelem = ET.SubElement(elem,child['elem'])
            if child.has_key('text') and child['text']:
                subelem.text = child['text']
            if child.has_key('attrib') and child['attrib']:
                subelem.attrib = child['attrib']
            for ch in child['children']: _traverse_element(ch,subelem)
            return

        machine = ET.Element('machine',id=self.id)          # id 
        for type,set in {'nickname':self.nickname, 'fqdn':self.fqdn,\
                         'bootargs':self.bootargs}.items():
            ET.SubElement(machine,type).text = set
        for i in self.archlist:                             # arch(es)
            ET.SubElement(machine,'arch').text = i
        for i in self.fingerprints:                         # fingerprints
            ET.SubElement(machine,'fingerprint').text = i
        elem = ET.SubElement(machine,'ksdevice').text = self.ksdevice
        for i in self.hwinfo:                               # hwinfo
            elem = ET.SubElement(machine,i['elem'])
            if i.has_key('text') and i['text']:
                elem.text = i['text']
            if i.has_key('attrib') and i['attrib']:
                elem.attrib = i['attrib']
            for child in i['children']:
                _traverse_element(child,elem)
        return machine
 
    def xml(self):
        return ET.tostring(indent(self.element()))
